@@ -234,7 +234,7 @@ static void
 setup_gui (ECategoriesMasterListDialog *ecmld)
 {
 	GladeXML *gui = glade_xml_new (
-		GAL_GLADEDIR "/e-categories-master-list-dialog.glade", NULL, PACKAGE);
+		GAL_GLADEDIR "/e-categories-master-list-dialog.glade", NULL, E_I18N_DOMAIN);
 	GtkWidget *dialog;
 
 	ecmld->priv->gui = gui;
