@@ -82,17 +82,17 @@ char *headers [COLS] = {
 #endif
 
 #define SPEC "<ETableSpecification cursor-mode=\"line\" selection-mode=\"browse\" draw-focus=\"true\" allow-grouping=\"false\">				\
-  <ETableColumn model_col=\"0\" _title=\"Email\"   expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\" compare=\"string\"/> \
-  <ETableColumn model_col=\"1\" _title=\"Full Name\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
-  <ETableColumn model_col=\"2\" _title=\"Address\"     expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
-  <ETableColumn model_col=\"3\" _title=\"Phone\"      expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
-        <ETableState> \
-	        <column source=\"0\"/> \
-	        <column source=\"1\"/> \
-	        <column source=\"2\"/> \
-	        <column source=\"3\"/> \
-	        <grouping> <leaf column=\"1\" ascending=\"true\"/> </grouping>    \
-        </ETableState> \
+  <ETableColumn model_col=\"0\" _title=\"Email\"   expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"     compare=\"string\"/>	\
+  <ETableColumn model_col=\"1\" _title=\"Full Name\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"   compare=\"string\"/>	\
+  <ETableColumn model_col=\"2\" _title=\"Address\"     expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\" compare=\"string\"/>	\
+  <ETableColumn model_col=\"3\" _title=\"Phone\"      expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"  compare=\"string\"/>	\
+        <ETableState>																	\
+	        <column source=\"0\"/>															\
+	        <column source=\"1\"/>															\
+	        <column source=\"2\"/>															\
+	        <column source=\"3\"/>															\
+	        <grouping> <leaf column=\"1\" ascending=\"true\"/> </grouping>										\
+        </ETableState>																	\
 </ETableSpecification>"
 
 /*
@@ -194,6 +194,12 @@ make_rand_str (void)
 	return g_strdup_printf ("%d%2x", i++, rand ());
 }
 
+static void
+weak_ref_func (gpointer data, GObject *where_object_was)
+{
+	gtk_main_quit();
+}
+
 /* We create a window containing our new table. */
 static void
 create_table (void)
@@ -245,6 +251,8 @@ create_table (void)
 
 	/* Show it all. */
 	gtk_widget_show_all (window);
+
+	g_object_weak_ref (window, weak_ref_func, NULL);
 }
 
 /* This is the main function which just initializes gnome and call our create_table function */
